#ifdef _PROTOTYPE_

#include "common.hh"
#include "image.hh"
#include "font.hh"

class TheDisplay
{
public:
    int w, h, bpp, hz, flags;

    static const int DISPLAY_WINDOWED = 1;
    static const int DISPLAY_FULLSCREEN = 2;

    TheDisplay::TheDisplay(int w_, int h_, int bpp_, int hz_, int flags_)
    {
        this->w = w_;
        this->h = h_;
        this->bpp = bpp_;
        this->hz = hz_;
        this->flags = flags_;
        set();
    }

    static void flip()
    {
        allegro_gl_flip();
    }

    void set();
    void toggle_fullscreen();
};

#endif /* _PROTOTYPE_ */

#include "display.hh"

void TheDisplay::toggle_fullscreen()
{
    TheDisplay *self = this;
    int cd = desktop_color_depth();
    if (self->bpp)
        cd = self->bpp;

    set_color_depth(cd);
    if (self->hz)
        request_refresh_rate(self->hz);

    allegro_gl_set(AGL_COLOR_DEPTH, bpp);
    allegro_gl_set(AGL_SUGGEST, AGL_COLOR_DEPTH);

    allegro_gl_set(AGL_DOUBLEBUFFER, 1);
    allegro_gl_set(AGL_SUGGEST, AGL_DOUBLEBUFFER);

    allegro_gl_set(AGL_SAMPLE_BUFFERS, 0);
    allegro_gl_set(AGL_SUGGEST, AGL_SAMPLE_BUFFERS);

    allegro_gl_set(AGL_SAMPLES, 0);
    allegro_gl_set(AGL_SUGGEST, AGL_SAMPLES);

    allegro_gl_set(AGL_RENDERMETHOD, 1);
    allegro_gl_set(AGL_SUGGEST, AGL_RENDERMETHOD);

    int mode = GFX_OPENGL;
    if (flags & DISPLAY_WINDOWED)
    {
        mode = GFX_OPENGL_WINDOWED;
        flags ^= DISPLAY_WINDOWED;
        flags ^= DISPLAY_FULLSCREEN;
    }
    else if (flags & DISPLAY_FULLSCREEN)
    {
        mode = GFX_OPENGL_FULLSCREEN;
        flags ^= DISPLAY_FULLSCREEN;
        flags ^= DISPLAY_WINDOWED;
    }

    set_gfx_mode(mode, w, h, 0, 0);

    glDisable(GL_CULL_FACE);
    glDisable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, SCREEN_W, 0, SCREEN_H, -1, 1);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glClearColor(0, 0, 0, 0);

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    Image::reload();
    TheFont::reload();
}

void TheDisplay::set()
{
    if (install_allegro_gl())
    {
        throw "Install_allegro_gl failed!";
    }
    toggle_fullscreen();
}

