#ifdef _PROTOTYPE_

#include "common.hh"

class Image
{
public:
    Image(char const *filename);
    void draw_scaled_rotated_tinted(float x, float y, float sx, float sy, float a, float r, float g, float b);
    void draw(float x, float y);
    void draw_tinted(float x, float y, float r, float g, float b);

    char *filename;
    char *name;
    BITMAP *memory_cache;
    int gl_texture;

    float xoff, yoff;

    Image *next;
    static Image *first;

    static void reload()
    {
        Image *img;
        for (img = first; img; img = img->next)
        {
            img->reload_me();
        }
    }

    void reload_me();
};
#endif /* _PROTOTYPE_ */

#include "image.hh"

Image *Image::first = NULL;

void Image::reload_me()
{
    set_color_conversion(COLORCONV_NONE);
    if (memory_cache)
        destroy_bitmap(memory_cache);
    BITMAP *bmp = load_bitmap(filename, NULL);
    if (bmp)
    {
        BITMAP *temp = create_bitmap_ex(32, bmp->w, bmp->h);
        blit(bmp, temp, 0, 0, 0, 0, bmp->w, bmp->h);
        int y;
        for (y = 0; y < bmp->h; y++)
        {
            blit(temp, bmp, 0, bmp->h - 1 - y, 0, y, bmp->w, 1);
        }
        destroy_bitmap(temp);

        this->memory_cache = bmp;

        allegro_gl_set_texture_format(GL_RGBA8);
        this->gl_texture = allegro_gl_make_texture(this->memory_cache);

        this->xoff = 0;
        this->yoff = 0;
    }
}

Image::Image(char const *filename_)
{
    this->filename = strdup(filename_);
    this->name = strdup(filename);
    this->memory_cache = NULL;

    reload_me();

    this->next = first;
    first = this;
}

static void quad(Image *self)
{
    glBindTexture(GL_TEXTURE_2D, self->gl_texture);
    glBegin(GL_TRIANGLE_FAN);

    GLfloat w = self->memory_cache->w;
    GLfloat h = self->memory_cache->h;

    GLfloat l = 0;
    GLfloat t = 0;
    GLfloat r = self->memory_cache->w;
    GLfloat b = self->memory_cache->h;

    GLfloat mx = l + self->xoff;
    GLfloat my = t + self->yoff;

    l += 0;
    t += 0;
    r -= 0;
    b -= 0;

    glTexCoord2f(r / w, (h - b) / h);
    glVertex2d(r - mx, b - my);

    glTexCoord2f(l / w, (h - b) / h);
    glVertex2d(l - mx, b - my);

    glTexCoord2f(l / w, (h - t) / h);
    glVertex2d(l - mx, t - my);

    glTexCoord2f(r / w, (h - t) / h);
    glVertex2d(r - mx, t - my);

    glEnd();
}

void Image::draw_scaled_rotated_tinted(float x, float y,
    float sx, float sy, float a, float r, float g, float b)
{
    glColor3f(r, g, b);
    glPushMatrix();
    glTranslatef(x, y, 0);
    glScalef(sx, sy, 1);
    glRotatef(a * 180 / AL_PI, 0, 0, 1);
    quad(this);
    glPopMatrix();
}

void Image::draw(float x, float y)
{
    draw_scaled_rotated_tinted(x, y, 1, 1, 0, 1, 1, 1);
}

void Image::draw_tinted(float x, float y, float r, float g, float b)
{
    glColor3f(r, g, b);
    draw_scaled_rotated_tinted(x, y, 1, 1, 0, 1, 1, 1);
}
