#ifdef _PROTOTYPE_

#include "common.hh"
#include "display.hh"

class Runner
{
public:
    char *name;
    Runner(char const *name_)
    {
        name = strdup(name_);
    }
    virtual void enter() {}
    virtual void tick() {}
    virtual void draw() {}
    virtual void leave() {}
    virtual ~Runner() {}

    static Runner *active;
    void switch_to()
    {
        if (active)
            active->leave();
        active = this;
        clear_keybuf();
        active->enter();
    }

    static void run(int frequency = 60);

    static volatile int ticks;
    static int frames;
    static int gframes;
    static int quit;
    static void ticker();
};

#endif /* _PROTOTYPE_ */

#include "runner.hh"

Runner *Runner::active = NULL;
int Runner::frames = 0;
int Runner::gframes = 0;
int Runner::quit = 0;

volatile int Runner::ticks = 0;
void Runner::ticker()
{
    ticks++;
}

void Runner::run(int frequency)
{
    install_int_ex(ticker, BPS_TO_TIMER(frequency));

    frames = ticks;
    gframes = frames;
    while (!quit)
    {
        while (frames <= ticks)
        {
            if (active)
                active->tick();
            frames++;
        }
        if (gframes < frames)
        {
            if (active)
                active->draw();
            TheDisplay::flip();
            gframes = frames;
        }
        else
        {
            rest(1);
        }
    }
}
